def run():
	
	import utime as time
	import uasyncio as asyncio
	import machine

	from configure import *
	from lcd_i2c_printer import *
	from urbanbus import bus_service
	    
	text_added      = 'Powered by welbits.com'

	# Read de configuration files                       
	config          = read_config_file()
	config_client   = config['configuration']['network']['client']
	stopid          = config['settings']['stopid']

	# Boot message sequence
	name  = config['about']['name']
	model = config['about']['model']

	print_on_lcd(name)
	time.sleep(1)
	print_on_lcd(name, 'Model: '+ model)
	time.sleep(2)
	moving_message(text_added,
	           1,
	           0.2,
	           config['about']['name'].center(I2C_NUM_COLS),
	           4.8)

	# Attempting to connect network

	waiting_message('Conectando Wi-Fi')

	try:
	wifi_client = start_wifi(config_client['ssid'],
	                         config_client['password'],
	                         'client')

	print("Status: ",wifi_client.status())

	if wifi_client.status() != 3: # Trigger setup
	    raise OSError('ssid is empty')

	else: # Allow change stop id during x time
	    
	    waiting_message("Elija parada")
	    time.sleep(2)
	    
	    init_server('/save_stopid')
	    task1 = waiting_for_config(60, 'IP: ' + wifi_client.ifconfig()[0])
	    task2 = asyncio.start_server(serve_client, '0.0.0.0', 80)
	    
	    asyncio.run(asyncio.gather(task1, task2))
	    
	    stopid = read_config_file()['settings']['stopid']
	    
	except OSError: # As can't connect let entry to setup page

	wifi_connected = False

	while not wifi_connected:
	    waiting_message('Configurando')
	    
	    net_name        = config['configuration']['network']['ap']['ssid']
	    psswrd          = config['configuration']['network']['ap']['password']
	    
	    access_point    = start_wifi(net_name, psswrd, 'ap')

	##################################################################33
	    
	    init_server('/save_wifi', access_point.scan())
	    
	    #           0123456789ABCDEF
	    msg     = ['Configure su ',
	               'dispositivo:',
	               'Conecte a SSID' ,
	               config['about']['name'],
	               'con contrase\356a:',
	               config['configuration']['network']['ap']['password'],
	               'Navegue a:',
	               access_point.ifconfig()[0]
	              ]
	           
	    task1 = info_during_setup(msg)
	    task2 = asyncio.start_server(serve_client, '0.0.0.0', 80)

	    tasks = asyncio.gather(task1,task2)
	        
	    asyncio.run(tasks)                        

	    waiting_message('Conectando Wi-Fi')        
	#####################################################################       
	    print(access_point.ifconfig())   

	    time.sleep(3)

	    access_point.active(False); access_point.deinit(); del access_point
	      
	    config_client = read_config_file()['configuration']['network']['client']
	    time.sleep(3)
	    wifi_client   = start_wifi(config_client['ssid'],
	                               config_client['password'],
	                               'client')
	    
	    if wifi_client.status() == 3:
	        wifi_connected = True
	        
	    wifi_client.status()
	    print ('Status: ' + str(wifi_client.status()))

	stopid = read_config_file()['settings']['stopid']

	waiting_message('Buscando datos')
	 
	try : #Para la fase de desarrollo
	bus_service()


	finally:
	wifi_client.active(False); wifi_client.deinit() ; del wifi_client #No conecta AP en la segunda ejecución



def url(stopid): 
    return 'https://api.interurbanos.welbits.com/v1/stop/' + stopid


#Call to API and get json data
def call_for_data(url):
    try:
        import urequests as requests
    except ImportError:
        import requests
    r           = requests.get(url = url)
    json_data   = r.json() #r.text.json()
    status_code = r.status_code
    #print('Testeo si ejecuta r.close')
    r.close()
    return json_data, status_code


def how_much_to(value):
    import utime as time
    
    h, m     = int(value[0]), int(value[1])
    now      = time.localtime()
    after    = list(now)

    if now[3] <= h:
        if now[4] <= m:
            after[2] = after[2] + 1
    after[3],after[4] = h, m
    
    return time.mktime(tuple(after)) - time.time()


#Filter for the next arrival for each line
def sel_next_bus_each_line(api_data):
    import utime as time
    
    next_bus = [] ; control = []
    
    for i in api_data['lines']:
        if i['lineNumber'] not in control:
            next_bus.append([i['lineNumber'],i['waitTime']])
            control.append(i['lineNumber'])
            if all(list(map(lambda a: ':' in a[1], next_bus))):
                next_bus.sort(key = lambda x: int(x[1].replace(':','')))
                first_bus_time = next_bus[0][1].split(':')
                delay = how_much_to(first_bus_time) - 30 * 60
                time.sleep(delay)
                
    return next_bus  # output: list


#Manage the behaivor according to the API response
def handle_api_responses(schedule, HTTPResponseCode): # Llamar con callForData(url(stopid))
    import utime as time
    from lcd_i2c_printer import show_wait_time, waiting_message, moving_message
    
    global count_500
    break_while = False
    
    if HTTPResponseCode == 200:
        count_500 = 0
        delay = 60

        arrivals = sel_next_bus_each_line(schedule)
        delay = show_wait_time(arrivals, delay)
                       
    elif HTTPResponseCode == 404:
        print("Connection Failed. Code: " + str(HTTPResponseCode))
        moving_message('Reinicie el dispositivo y configure parada',
           1,
           0.2,
           'Parada erronea!',
           10
           )
        break_while = True
        
    elif HTTPResponseCode == 500:
        
        print("Connection Failed. Code: " + str(HTTPResponseCode))
        
        if count_500 < 10:               
            delay = 10
                
        else:
        
            waiting_message('Server Error:   Espere')
            delay = 300
        
        count_500 += 1
    
    else:
        waiting_message('Error. Reinicie')
        break_while = True
           
    time.sleep(delay)
    return break_while


#Launch the bus service
def bus_service():
    from configure import read_config_file
    
    global count_500
    count_500 = 0
    stopid = read_config_file()['settings']['stopid']
    
    while True:
        schedule, HTTPResponseCode = call_for_data(url(stopid))
        break_while = handle_api_responses(schedule, HTTPResponseCode)
        if break_while == True:
            break

